// SmplFIFOPlay01.cpp : Defines the entry point for the console application.
#include "stdafx.h"
#include <windows.h>

#include "FTD2xx.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <memory.h>

unsigned long read4bytes(FILE *f)
{
    unsigned char buf[4];

    if (fread(buf, 4, 1, f) != 1) {
        fprintf(stderr, "Read error\n");
        exit(1);
    }
    return ((256LU * buf[3] + buf[2]) * 256LU + buf[1] ) * 256LU + buf[0]  ;
}

unsigned read2bytes(FILE *f)
{
    unsigned char buf[2];

    if (fread(buf, 2, 1, f) != 1) {
        fprintf(stderr, "Read error\n");
        exit(1);
    }
    return 256U * buf[1] + buf[0] ;
}

void WavToFifo(char *filename)
{
    FILE *f;
    int i, channels, bits;
    unsigned long len;
    long count;
    unsigned char s[10];


	f = fopen(filename, "rb");

    if (f == NULL) {
        printf("Can not open %s\n", filename);
        return;
    }
    printf("finename = '%s'\n", filename);
    if (fread(s, 4, 1, f) != 1) {
        printf("Read error\n");
        fclose(f);
        return;
    }
	
    if (memcmp(s, "RIFF", 4) != 0) {
        printf("Not a RIFF format\n");
        fclose(f);
        return;
    }

    printf("[RIFF] (%lu bytes)\n", read4bytes(f));
    if (fread(s, 8, 1, f) != 1) {
        printf("Read error\n");
        fclose(f);
        return;
    }
	
    if (memcmp(s, "WAVEfmt ", 8) != 0) {
        printf("Not a WAVEfmt format\n");
        fclose(f);
        return;
    }
    len = read4bytes(f);
    printf("[WAVEfmt ] (%lu bytes)\n", len);
    if (len != 16) {
        printf("Length of WAVEfmt must be 16\n");
        return;
    }
    printf("  Data type = %u (1 = PCM)\n", read2bytes(f));
    channels = read2bytes(f);
    printf("  Number of channels = %u (1 = mono, 2 = stereo)\n", channels);
    printf("  Sampling rate = %luHz\n", read4bytes(f));
    printf("  Bytes per second = %lu\n", read4bytes(f));
    printf("  Bytes per sample = %u\n", read2bytes(f));
    bits = read2bytes(f);
    printf("  Bits per sample = %u\n", bits);

    while (fread(s, 4, 1, f) == 1) 
	{
        len = read4bytes(f);
        s[4] = 0;
        printf("[%s] (%lu bytes)\n", s, len);
        if (memcmp(s, "data", 4) == 0) break;
        for (i = 0; i < (int)len; i++)
            printf("%02x ", fgetc(f));
        printf("\n");
    }

	//FTDI Related
	FT_STATUS ftStatus;
	FT_HANDLE ftHandle;
	DWORD dwToWrite = 128;
	DWORD dwWritten;

	ftStatus = FT_Open(0, &ftHandle);
	if (ftStatus == FT_OK)
	{
		//OK
	}
	else
	{
	// FT_Open failed
		printf("Can't open FTDI Device.\n");
		return;
	}
	ftStatus = FT_Purge(ftHandle, FT_PURGE_RX | FT_PURGE_TX);
	ftStatus = FT_SetTimeouts(ftHandle,1000,1000);
	ftStatus = FT_SetLatencyTimer(ftHandle,1);

	//get memory for "len" bytes
	unsigned char * p;
	p = (unsigned char *)malloc(len);

	fread(p, len, 1,f);

	dwToWrite = 61440;
	unsigned long pCounter = 0;

	for (count = 0; count < 0xFFFFFFFF; count++)
	{
		if (pCounter > len) goto loopend;

		if (FT_Write(ftHandle, (p+pCounter),  dwToWrite, &dwWritten) == FT_OK)
		{
			if ( dwToWrite == dwWritten)
			{
				//FT_W32_WriteFile OK
			}
			else
			{
				printf("FT_W32_WriteFile TimeOut\n");
			}
		}
		else
		{
			printf("FT_W32_WriteFile Failed\n");
			goto loopend;
		}
		pCounter += dwToWrite;
	}

 loopend:
    fclose(f);
	free(p);
	FT_Close(ftHandle);
}

int _tmain(int argc, char * argv[])
{
	WavToFifo(argv[1]);
    return 0;
}

